/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#ifndef APPUTIL_H
#define APPUTIL_H
#include "libraries/fstUtils.h"
using namespace fst;
using namespace std;

typedef set < pair <int, int> > Graph;

class block {
public:
	vector <string> topkpaths;
	vector <double> topkpathwts;
	double totprob;
	int minnode;			//the smallest node
	int maxnode;			//the largest node as per fst partial order
	block() {
		totprob = 0.0;
		minnode = maxnode = -1;
	}
	~block(){}
};

class blockskel {	//block skeleton, ie block without the paths/wts
public:
	Graph origfstedges;	//set of edges in the orig fst, this uniquely defines a subgraph and thus a block
	set <int> origfstnodes;
	int minnode;
	int maxnode;
	set < Graph > parskels;	//the origfstnodes set of this blockskel's immediate parents (skels)
	blockskel() {
		minnode = maxnode = -1;
	}
	~blockskel(){}
};

block getBlockData(StdVectorFst *ifst, blockskel thisskel, int numnodes, int K, int storepaths);
vector <blockskel> getNgbrSkels(blockskel thisskel, vector <blockskel>* allskels);
void fixMidlSkel(blockskel *thisskel, vector <blockskel> *allskels);
void fixMinMaxSkel(blockskel *thisskel, map <int, set<int> > adjlist, int* nodedismin, int numnodes, vector <blockskel> *allblockskels, int minmax);
blockskel getMergedSkel(blockskel chosencandskel, blockskel ngbr, map <int, set<int> > adjlist, int* nodedismin, int numnodes, vector <blockskel> *allblockskels);

map < Graph, vector <Graph > > getBlockSkelGraph(vector <blockskel> *skels);
set <Graph > getParentSkels(Graph thisset, vector <blockskel> *allblockskels);
map <int, set <int> > adjlistFromSkels(vector <blockskel> *allblockskelsnew);
void updateStructures(blockskel thisblkskel, block thisblk, map < Graph, block> *allblocks, vector <blockskel> *allblockskels, map <int, set <int> > *adjlist, map < Graph, block> *allblocksnew, vector <blockskel> *allblockskelsnew, map <int, set <int> > *adjlistnew);

double probMultDyn(map < Graph, vector < Graph > >  *blkskelgraph, map < Graph, block> *allblocks, Graph startskelset, map < Graph, double> *memo);
double edgePushDynProb(vector <blockskel> *allblockskels, map < Graph, block> *allblocks);

set <int> intersectSets(set <int> s1, set <int> s2);
set < pair<int, int> > intersectSetsEdges(set < pair<int, int> > s1, set < pair<int, int> > s2);
set <int> uniteSets(set <int> s1, set <int> s2);
set < pair<int, int> > uniteSetsEdges(set < pair<int, int> > s1, set < pair<int, int> > s2);
int subsetCheck(set <int> s1, set <int> s2);
int subsetCheckEdges(set < pair<int, int> > s1, set < pair<int, int> > s2);

#endif
