/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#ifndef PATHENUMERATOR_H
#define PATHENUMERATOR_H

#include <fst/shortest-path.h>
#include <fst/vector-fst.h>
#include <fst/float-weight.h>
#include <fst/random-weight.h>
#include <fst/flags.h>
#include <fst/project.h>
#include <fst/fst.h>
#include <fst/arc.h>
#include <fst/map.h>
#include <fst/compose.h>
#include <fst/arcsort.h>
#include <fst/weight.h>
#include <fst/float-weight.h>

#include <sstream>
#include <algorithm>
#include <sstream>
#include <string>
#include <sys/time.h>
#include <queue>
#include <set>
#include <sys/time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <iostream>

#include "NodeEdge.h"
#include "Path.h"

using namespace std;
using namespace fst;

class Dist {
public:
    int index;
    double cost;
    EdgeWeight *edge;
    Dist &operator=(const Dist &rhs); 
};

// for heap in Yen's; priority ~ shorter; rhs higher ~ true
struct ComparePath {
	bool operator()(const Path &lhs, const Path &rhs) const {
		return lhs.getPathCost() > rhs.getPathCost();
	}
};

struct CompareDist {
    bool operator()(const Dist &lhs, const Dist &rhs) const {
    	return lhs.cost > rhs.cost;
    }
};

struct CompareEdgeWeight {
    bool operator()(const EdgeWeight &lhs, const EdgeWeight &rhs) const {
        return (lhs.from < rhs.from)
            || (lhs.from == rhs.from && lhs.to < rhs.to)
            || (lhs.from == rhs.from && lhs.to == rhs.to
                && lhs.label < rhs.label);
    }
};

class PathEnumerator {
public:
    vector<Path> A;
    string temppath;
    typedef StdArc Arc;
    typedef StdArc::StateId StateId;
    typedef StdArc::Label Label;
    typedef StdArc::Weight Weight;
    PathEnumerator(const Fst<StdArc> &fst, const SymbolTable *isyms, const SymbolTable *osyms, const SymbolTable *ssyms, bool accep, bool show_weight_one) 
    	: fst_(fst), isyms_(isyms), osyms_(osyms), ssyms_(ssyms), accep_(accep && fst.Properties(kAcceptor, true)), ostrm_(0), show_weight_one_(show_weight_one) {
        A_.clear();
        k_ = 0;
        is_first_time_ = true;
        shortest_to_sink_.clear();
        graph_data_.clear();
        edge_buffer_.clear();
    }
    ~PathEnumerator();

    // for yen's algo
    void fstToGraph();
    bool getNextBest(Path &);
    vector<Path> getBufferedPaths(int startpathno,int endpathno);
    void pathToString(Path& p); 
    void initShortestToSink();
    bool addPathToHeap(Path &, int, EdgeWeight *);

    const Fst<StdArc> &fst_;
  	const SymbolTable *isyms_;  // ilabel symbol table
  	const SymbolTable *osyms_;  // olabel symbol table
  	const SymbolTable *ssyms_;  // slabel symbol table
  	bool accep_;                // print as acceptor when possible
  	ostream *ostrm_;            // text FST destination
  	string dest_;               // text FST destination name
  	bool show_weight_one_;      // print weights equal to Weight::One()

    // for yen's
  	vector<Path> A_;
  	int k_;
		bool is_first_time_;
		vector<Path> shortest_to_sink_;
		Node *source_node_;
		Node *sink_node_;
		vector<Node*> graph_data_;
  	priority_queue<Path, vector<Path>, ComparePath> paths_heap_;
   	vector<EdgeWeight *> edge_buffer_;
		vector<Node*> graph_data_2_; // keep the shortest arc only
   	DISALLOW_COPY_AND_ASSIGN(PathEnumerator);
};
#endif
