/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#include "libraries/Path.h"
Path::Path() {
    this->str = "";
    this->edges.clear();
    this->cost = 0.0;
}

int Path::numOfEdges() const {
	return this->edges.size();
}

string Path::getPathStr() const {
	return this->str;
}

double Path::getPathCost() const {
	return this->cost;
}

EdgeWeight *Path::getEdge(int index) {
	return this->edges.at(index);
}

void Path::appendEdge(EdgeWeight *ew) {
    this->str += ew->label;
	this->edges.push_back(ew);
    this->cost += ew->weight;
}

void Path::appendPath(Path &p) {
    int i;
    for (i = 0; i < p.numOfEdges(); i ++) {
        this->appendEdge(p.getEdge(i));
    }
}

void Path::appendPath(Path &p, int begin, int end) {
    int i;
    for (i = begin; i < p.numOfEdges() && i < end; i ++) {
        this->appendEdge(p.getEdge(i));
    }
}

Path &Path::operator=(const Path &rhs) {
    this->str = rhs.str;
	this->edges.assign(rhs.edges.begin(), rhs.edges.end());
	this->cost = rhs.cost;
	return (*this);
}

void Path::setPathCost(double pathcost) {
	this->cost = pathcost;
}

Path::Path(string pathstring,double curpathcost) {
	this->str = pathstring;
	this->cost = curpathcost;
}

bool Path::operator<(const Path& a) {
	return this->cost < a.cost;
}

void Path::setPath(string input) {
	this->str = input;
}
