/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#include "libraries/regexParser.h"

//convert a given linear regex (no nestings) into query fst; returns -1 is pattern is unparsable, else 0 if queryfst is constructed successfully
int getQueryFst(StdVectorFst* query, char* reg) {
	int len = strlen(reg);
	int i = 0;
	query->AddState();	//add its corresp state
	query->SetStart(0);			
	int curst = 0;	
	query->AddState();	//add its corresp next state
	int start = -1;
	while(i < len) {
		if(reg[i] == '\\') {
			i++;
			//escape sequence starts
			if(reg[i] == 'd') {
				for(int n=48; n<58; n++) {
					query->AddArc(curst, StdArc(n, n, 0, curst + 1));
				}
			}
			else if(reg[i] == 'a') {
				for(int n=65; n<91; n++) {
					query->AddArc(curst, StdArc(n, n, 0, curst + 1));
				}
				for(int n=97; n<123; n++) {
					query->AddArc(curst, StdArc(n, n, 0, curst + 1));
				}
			}
			else if(reg[i] == 'x') {
				for(int n=32; n<126; n++) {
					query->AddArc(curst, StdArc(n, n, 0, curst + 1));
				}
			}
			else if((reg[i] == '|') || (reg[i] == '(') || (reg[i] == ')') || (reg[i] == '\\')) {	//(|)\ chars
				query->AddArc(curst, StdArc(reg[i] - '\0', reg[i] - '\0', 0, curst + 1));
			}
			else {
				cout << "Error: Unparsable regex pattern\n";
				return -1;
			}
			curst++;
			query->AddState();	//add its corresp next state
		}
		else if(reg[i] == '(') {
			if(start < 0)
				start = curst;
			else {
				cout << "Error: Unparsable regex pattern\n";
				return -1;
			}				
		}
		else if(reg[i] == '*') {	//the kleene star - remove the latest st, and put a self loop at prev
				if(curst > 0) //go to prev state:
					curst--;
				vector<int> arclabels; //get the arcs out of that state, and resend them to the same state!
				for(ArcIterator< Fst<StdArc> > aiter(*query, curst);!aiter.Done(); aiter.Next()) {
					StdArc arc = aiter.Value();
					arclabels.push_back(arc.ilabel);
				}
				query->DeleteArcs(curst);
				for(int i=0; i< arclabels.size(); i++) {
					query->AddArc(curst, StdArc(arclabels[i], arclabels[i], 0, curst));
				}		
		}
		else if(reg[i] == ')') {
			start = -2;
		}
		else if(reg[i] == '|') {
			if(start > -2) {
				curst--;
			}
			else if (start == -2) {
				cout << "Error: Unparsable regex pattern\n";
				return -1;
			}
		}
		else if(((reg[i] - '\0') >= 32) && ((reg[i] - '\0') <= 125)) {
			query->AddArc(curst, StdArc(reg[i] - '\0', reg[i] - '\0', 0, curst + 1));
			curst++;
			query->AddState();	//add its corresp next state
		}
		else{
				cout << "Error: Unparsable regex pattern\n";
			return -1;
		}
		i++;
	}
	query->SetFinal(curst, 0);
	Connect(query);	//to remove states after final state
	ArcSort(query, ILabelCompare<StdArc>()); //sort on ilabels
	return 0;
}
