/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <sstream>
#include <vector>
#include <cstdlib>
#include "libpq-fe.h"
using namespace std;

int main(int argc, char** argv) {
	if(argc < 7) {
		cout << "Usage: loadkmap <dbname> <port number> <kmap table> <docname> <mbinname> <.kmap full filepath>\n";
		return 0;
	}
	PGconn* conn;
	string dbstring = "dbname=" +  string(argv[1]) + " port=" + string(argv[2]);
	conn = PQconnectdb(dbstring.c_str());
	if(PQstatus(conn) != CONNECTION_OK) {
		cout << "Error: Could not connect with " << dbstring << "! Message: " << PQerrorMessage(conn);
		PQfinish(conn);
		exit(1);
	}
	ifstream fin(argv[6], ios::in);
	string tablename = string(argv[3]);
	string docname = string(argv[4]);
	string fstname = string(argv[5]);
	string pathkey = docname + "-" + fstname;
	vector <string> paths;
	vector <string> pathwts;
	string comnd = "insert into " + tablename + " values ";
	while(!fin.eof()) {
		string kmapline;
		getline(fin, kmapline);
		if(kmapline.compare("") == 0)	//empty string
			continue;
		char* tok1 = strtok((char*)kmapline.c_str(), "\t");	//removing the \t:\t between the maptext and weightscore; first path
		vector <string> toks1;
		while(tok1 != NULL) {
			toks1.push_back(tok1);
			tok1 = strtok(NULL, "\t");
		}
		string path = toks1[0];
		string pathwt = toks1[2];
		int slen = path.length();
		char tostr[2*slen+1];
		memset(tostr, '\0', 2*slen+1);
		int erro = 0;
		PQescapeStringConn(conn, tostr, path.c_str(), slen, &erro);
		if(erro != 0) {
			cout << "Error: Cannot escape string " << path << " and thus not inserting!" << endl;
			exit(1);
		}
		paths.push_back(string(tostr));
		pathwts.push_back(pathwt);
	}
	stringstream pathnum;
	pathnum << 0;
	comnd += "('" + pathkey + "', " + pathnum.str() + ", E'" + paths[0] + "', " + pathwts[0] + ")";
	for(int i=1; i<paths.size(); i++) {
		stringstream pathnum;
		pathnum << i;
		comnd += ", ('" + pathkey + "', " + pathnum.str() + ", E'" + paths[i] + "', " + pathwts[i] + ")";
	}
	comnd += ";";
	PGresult *res;
	res = PQexec(conn, comnd.c_str());
	PQclear(res);
	PQfinish(conn);
return 0;
}
