/*
Copyright 2012 Arun Kumar and Christopher Re

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

This work was supported by the Microsoft Jim Gray Systems Lab, 
the National Science Foundation and the Office of Naval Research.
*/
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <sstream>
#include <vector>
#include <cstdlib>
#include <map>
#include "frontends/staccatoSQLUtils.h"

void dyndfsvisit(map <int, vector <int> >* revgph, structureInfo* stacinfo, int curblk, int* curlabel) {
	if((*revgph)[curblk].size() == 0) { //if no parent, apply label; and set reachprob to 1 and return
		(stacinfo->blkreachprobs)[curblk] = 1.0;
		(stacinfo->toplabs)[curblk] = *curlabel;
		*curlabel += 1;
		return;
	}
	double thisreachprob = 0.0;
	//call each parent only if doesnt have a label; in any case, get the par's reach prob and blkpob, and mult, and sum across all aprents to get this blk's reach prob
	for(vector<int>::iterator iter = (*revgph)[curblk].begin(); iter != (*revgph)[curblk].end(); iter++) {
		if((stacinfo->toplabs)[*iter] == -1)	//this par not visted before, so legal reachprob can be used only after visit
			dyndfsvisit(revgph, stacinfo, *iter, curlabel);
		thisreachprob += ((stacinfo->blkprobs)[*iter] * (stacinfo->blkreachprobs)[*iter]); //now that the par has been fin visted, we can use its memod prob
	}
	(stacinfo->blkreachprobs)[curblk] = thisreachprob; //memoize this blk's rpob and apply label
	(stacinfo->toplabs)[curblk] = *curlabel;
	*curlabel += 1;
}

int main(int argc, char** argv) {
	if(argc < 3) {
		cout << "Usage: graphgen <.stac file> <.graph file>\n";
		return 0;
	}
	ifstream fin(argv[1], ios::in);
	int M, K, numblks;
	string line1, line2;
	getline(fin, line1);
	getline(fin, line2);
	stringstream line1str(line1);
	stringstream line2str(line2);
	stringstream Mstr, Kstr;
	line1str >> M >> K;
	line2str >> numblks;
	Mstr << M;
	Kstr << K;		
	structureInfo stacinfo;
	for(int i=0; i<numblks; i++) {
		string gphline;
		getline(fin, gphline);
		char* linetok = strtok((char*)gphline.c_str(), "\t");
		int thisvx = atoi(linetok);
		vector <int> emptyv;
		stacinfo.stacgraph[thisvx] = emptyv;
		linetok = strtok(NULL, "\t");
		while((linetok != NULL)&&(strcmp(linetok, "") != 0)) {
			stacinfo.stacgraph[thisvx].push_back(atoi(linetok));
			linetok = strtok(NULL, "\t");
		}
		stacinfo.blkprobs.push_back(0.0);
		stacinfo.blkreachprobs.push_back(0.0);
		stacinfo.toplabs.push_back(0);
		stacinfo.toplabsrev.push_back(0);
	}
	for(int b=0; b<numblks; b++) {
		int blknum, tmp, numpaths;
		string linet;
		getline(fin, linet);
		stringstream linetstr(linet);
		linetstr >> blknum >> tmp >> numpaths;	//the blknum need not be b
		stacinfo.mmlens[blknum] = make_pair(1e6, 0);	//a large min, 0 is max on initlzn
		stacinfo.blkprobs[blknum] = 0.0;
		for(int p=0; p<numpaths; p++) {
			string path, pathwt;
			getline(fin, path);
			getline(fin, pathwt);
			double wt = atof(pathwt.c_str());
			stacinfo.blkprobs[blknum] += exp(-1.0 * wt);
			if(path.length() < stacinfo.mmlens[blknum].first)
				stacinfo.mmlens[blknum].first = path.length();
			if(path.length() > stacinfo.mmlens[blknum].second)
				stacinfo.mmlens[blknum].second = path.length();
		}
	}
	//write out the .graph binary file
	map <int, vector <int> > stacgraphrev;
	vector <int> endblks;
	for(int i=0; i<numblks; i++)
		stacgraphrev[i];
	for(map <int, vector <int> >::iterator iter = stacinfo.stacgraph.begin(); iter != stacinfo.stacgraph.end(); iter++) {
		for(vector <int>::iterator siter = iter->second.begin(); siter != iter->second.end(); siter++)
			stacgraphrev[*siter].push_back(iter->first);
		if(iter->second.size() == 0)
			endblks.push_back(iter->first);
	}
	for(int f=0; f<numblks; f++) {
		stacinfo.blkreachprobs[f] = 0.0;
		stacinfo.toplabs[f] = -1;		//-1 means not labeled yet!
	}
	int curlabel = 0;			//starting with this blkno label
	for(vector<int>::iterator siter = endblks.begin(); siter != endblks.end(); siter++)
		dyndfsvisit(&stacgraphrev, &stacinfo, *siter, &curlabel);
	string gphname = string(argv[2]);
	ofstream fout(gphname.c_str(), ios::out|ios::binary);
	graphwrite(&stacinfo, fout);
return 0;
}
